Attribute VB_Name = "modGradient"
Option Explicit

' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@
'   Types
' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@
Public Type RECT
    Left    As Long
    Top     As Long
    Right   As Long
    Bottom  As Long
End Type

' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@
'   Functions
' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@
Private Declare Function GetSysColor Lib "user32" (ByVal nIndex As Long) As Long
Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Private Declare Function Rectangle Lib "gdi32" (ByVal hdc As Long, ByVal x1 As Long, ByVal Y1 As Long, ByVal x2 As Long, ByVal Y2 As Long) As Long
Private Declare Function CreatePen Lib "gdi32" (ByVal nPenStyle As Long, ByVal nWidth As Long, ByVal crColor As Long) As Long
Private Declare Function CreateSolidBrush Lib "gdi32" (ByVal crColor As Long) As Long

' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@
'   Subs
' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@

' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@
'   Constants
' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@

' Color
Private Const COLOR_ACTIVECAPTION As Long = 2
' Create Pen
Public Const PS_SOLID = 0
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'   DrawGradientEffect :  Draws a gradient Effect across an area
'   specified by udtRegion
'       Parameters :
'           gradHDC         : the hDc to draw on
'           udtRect         : RECT structure that the gradient
'                   effect will be done one
'           lngStartColorColor   : the beginning Color, if not specified
'                   than the ActiveCaptionColor will be used
'           lngEndColorColor     : the ending color, if not specified
'                   then the Black will be used
'       Returns :
'           Nothing
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
Public Sub DrawGradientEffect(gradHdc As Long, _
        udtRegion As RECT, _
        lngStartColor As Long, _
        lngEndColor As Long)
    
    ' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@
    ' Dim all of our variables
    ' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@

    
    Dim lngPiece        As Long     ' The size of each piece to draw
    Dim lngCounter      As Long     ' A Counter for what piece we are drawing
    Dim lngPieceColor   As Long     ' the color of the drawn piece
    Dim rStep           As Long     ' The Value to increase the color by
    Dim gStep           As Long     ' The Value to increase the color by
    Dim bStep           As Long     ' The Value to increase the color by
    Dim r               As Long     ' The Red value of the color
    Dim g               As Long     ' The Green value of the color
    Dim b               As Long     ' The Blue value of the color
    
    ' Break up the region into 255 pieces
    Dim lngSteps        As Long     ' How many pieces we can break up the fade into
    
    ' GDI Objects
    Dim lOldBrush       As Long     ' The Old Brush before the Gradient Draw
    Dim lOldPen         As Long     ' The Old Pen before the Gradient Draw
    Dim lBrush          As Long     ' The New Brush for the Gradient Draw
    Dim lPen            As Long     ' The New Pen for the Gradient Draw
    
    
    ' Drawing Coordinates for the RGB Faded Color
    Dim x1              As Long     ' The X1 Coordinate to Draw the Gradient
    Dim x2              As Long     ' The X2 coordinate to Draw the Gradient

    Dim lReturn         As Long     ' The API return values
    
    ' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@
    ' Calculate the piece count
    ' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@
    Select Case (udtRegion.Right - udtRegion.Bottom)
        Case Is < 16
            lngSteps = 1
        Case Is < 32
            lngSteps = 16
        Case Is < 64
            lngSteps = 32
        Case Is < 128
            lngSteps = 64
        Case Is < 255
            lngSteps = 128
        Case Is < 255
            Exit Sub
        Case Else
            lngSteps = 255
    End Select
        
    ReDim RedColor(lngSteps) As Single    ' The Array of Red Colors to give the gradient effect
    ReDim GreenColor(lngSteps) As Single  ' The Array of Green Colors to give the gradient effect
    ReDim BlueColor(lngSteps) As Single   ' The Array of Blue Colors to give the gradient effect
        
    
    
    ' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@
    ' Calculate the RGB Faded Colors
    ' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@
    
    ' Calculate the Size of the step for each of red, green, blue
    ' color components
    rStep = ((lngEndColor And &HFF&) - (lngStartColor And &HFF)) / lngSteps
    gStep = (((lngEndColor And &HFF00&) - (lngStartColor And &HFF00&)) / lngSteps) / &H100&
    bStep = (((lngEndColor And &HFF0000) - (lngStartColor And &HFF0000)) / lngSteps) / &H10000
    
    ' Get the starting entries
    r = CByte(lngStartColor And &HFF)
    g = CByte(((lngStartColor And &HFF00&) \ &H100&) And &HFF)
    b = CByte(((lngStartColor And &HFF0000) \ &H10000) And &HFF)
    
    ' ***************************************************
    ' Draw the Gradient Effect
    ' ***************************************************
    
    ' Get the size of the pieces
    lngPiece = (udtRegion.Right - udtRegion.Left) \ lngSteps
    ' Fill in the palette entries
    For lngCounter = 0 To (lngSteps - 1)
        RedColor(lngCounter) = CByte(r)
        GreenColor(lngCounter) = CByte(g)
        BlueColor(lngCounter) = CByte(b)
        
        ' Convert the RGB into a number
        lngPieceColor = RGB(RedColor(lngCounter), GreenColor(lngCounter), BlueColor(lngCounter))
        
        'Create a Pen this color
        lPen = CreatePen(PS_SOLID, 1, lngPieceColor)
        lOldPen = SelectObject(gradHdc, lPen)
        
        ' Create the Solid Brush
        lBrush = CreateSolidBrush(lngPieceColor)
        lOldBrush = SelectObject(gradHdc, lBrush)
        
        x1 = udtRegion.Left + (lngCounter * lngPiece)
        x2 = udtRegion.Left + (lngCounter * lngPiece) + lngPiece
        
        ' Draw a rectangle with the lngPieceColor
        lReturn = Rectangle(gradHdc, x1, udtRegion.Top, x2, udtRegion.Bottom)
        
        ' Select back in the old brushes
        lReturn = SelectObject(gradHdc, lOldBrush)
        lReturn = SelectObject(gradHdc, lOldPen)
        ' Destory the Objects
        lReturn = DeleteObject(lBrush)
        lReturn = DeleteObject(lPen)
        
        ' Increment the color
        r = r + rStep
        g = g + gStep
        b = b + bStep
    Next lngCounter
    
    ' Get the Excess
    RedColor(lngCounter) = CByte(r - rStep)
    GreenColor(lngCounter) = CByte(g - gStep)
    BlueColor(lngCounter) = CByte(b - bStep)
    
    ' Convert the RGB into a number
    lngPieceColor = RGB(RedColor(lngSteps), GreenColor(lngSteps), BlueColor(lngSteps))
    
    'Create a Pen this color
    lPen = CreatePen(PS_SOLID, 1, lngPieceColor)
    lOldPen = SelectObject(gradHdc, lPen)
    
    ' Create the Solid Brush
    lBrush = CreateSolidBrush(lngPieceColor)
    lOldBrush = SelectObject(gradHdc, lBrush)
    
    x1 = udtRegion.Left + (lngSteps * lngPiece)
    x2 = udtRegion.Right
    
    ' Draw a rectangle with the lngPieceColor
    lReturn = Rectangle(gradHdc, x1, udtRegion.Top, x2, udtRegion.Bottom)
    
    ' Select back in the old brushes
    lReturn = SelectObject(gradHdc, lOldBrush)
    lReturn = SelectObject(gradHdc, lOldPen)
    ' Destory the Objects
    lReturn = DeleteObject(lBrush)
    lReturn = DeleteObject(lPen)


End Sub


